#include <p18f4550.h>
#include <i2c.h>	/*Gestion de la communication sur le bus I2C*/



/*Configuration du PIC*/
#pragma config BOR = ON



/*Bits de configuration, indpendant du programme, voir  Help->Topics->PicC18 config settings */

#pragma config MCLRE = OFF	/*Dsactiver la pin de Reset*/
#pragma config LVP = OFF	/*Low Voltage Programming dsactiv => RB5 I/O normale*/
#pragma config PBADEN = OFF	/*PORTB<4> and PORTB<1:0> Configured as Digital I/O Pins on Reset*/
#pragma config DEBUG = OFF	/*Dbug dsactiv => RB6 et RB7 normal I/O*/
#pragma config FCMEN = OFF	/*Fail-Safe Clock dsactive*/
#pragma config IESO = OFF	/*Oscillator Switchover mode dsactiv*/
#pragma config PWRT = OFF	/*Power-up Timer dsactiv*/
#pragma config WDT = OFF	/*Watchdog dsactiv*/
#pragma config WDTPS = 1024	/*Prdiviseur du watchdog*/
#pragma config LPT1OSC = OFF	/*Low-Power Timer 1 Oscillator Disable ==> high power*/
#pragma config XINST = OFF	/*Instruction set extension and Indexed Addressing mode disabled (Legacy mode)*/
#pragma config STVREN = OFF	/*Pas de reset du  la pile*/



#pragma config FOSC = INTOSCIO_EC



#define AdresseCommande1 0x40	/*Adresse de commande du premier circuit intgr PCF8574, reli aux broches  DB0    DB7  de l'afficheur alphanumrique*/
#define AdresseCommande2 0x42	/*Adresse de commande du deuxieme circuit intgr PCF8574, reli aux broches  R/S ,  R/W  et  E  de l'afficheur alphanumrique et aux 5 LED*/



void Delai(unsigned long temps);
unsigned char Ecrire(unsigned char adresse, unsigned char donnee);
unsigned char Lire(unsigned char adresse);
void EnvoiCommande(unsigned char commande);
void PlacerCurseur(unsigned char AdresseDuCurseur);
void EnvoiDonnee(unsigned char Donnee);
void InitialisationAfficheur();
void MiseVeilleAfficheur();
unsigned char ObtenirAdresseActuelle(void);
void AfficheCaractere(unsigned char c);
void AfficheTexte(unsigned short char * texte);
void CommandeSpeciale(unsigned char uoct_special);
void ChargeCaracteres(unsigned char str_nouveaucaractere[]);
void ConvertirEtAfficher(int valeur, unsigned char base);
void AllumerUneLED(unsigned char l);
void EteindreUneLED(unsigned char r);
void main(void);



unsigned short char Texte1[] = {'J', 'e', ' ', 's', 'u', 'i', 's', ' ', 'T', 'i', 'g', 'r', 'o', 'u', ' ', '!'};
unsigned short char Texte2[] = {'B', 'o', 'n', 'j', 'o', 'u', 'r', ' ', '!', ' ', ' ', ' ', ' ', ' ', ' ', ' '};
unsigned short char Texte3[] = {' ', ' ', ' ', 'A', 'u', 'd', 'i', 'o', 'v', 'o', 'r', 'e', ' ', ' ', ' ', ' '};

unsigned short char * TableauDeTextes[] = {Texte1, Texte2, Texte3};

unsigned char speciaux[56] = {0x0C, 0x12, 0x12, 0x0C, 0x00, 0x00, 0x00, 0x00,	/*Symbole   */
							  0x00, 0x0A, 0x15, 0x11, 0x11, 0x0A, 0x04, 0x00,	/*Symbole  Cur */ 
							  0x00, 0x00, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00,
							  0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 0x00, 0x00,
							  0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 0x00,
							  0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00,
							  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x00};

unsigned char AdresseCurseurActuelle;

unsigned char EtatDesLED;



/*Fonction qui sert  faire perdre son temps au STM32F100RB

Paramtre d'entre :

t - Valeur qui est proportionnelle au dlai que l'on souhaite obtenir.

Valeur de retour :

Aucune.

*/
void Delai(unsigned long temps)
	{
		unsigned long i;
			for(i = 0; i < temps; i++);
	}



/*Ecriture d'un octet grce au module  I2C  du PIC18F4550

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C , du composant que l'on souhaite crire,
donnees - Donne  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int Ecrire(unsigned char adresse, unsigned char donnee) 
   {
      int information = (-1);

         IdleI2C();    /*Attente de la disponibilit du bus I2C*/
         StartI2C();    /*Envoi de message de dbut de la communication I2C*/
         IdleI2C();    /*Attente de la disponibilit du bus I2C*/

         putcI2C(adresse); /*Envoi de l'adresse de l'esclave*/
         IdleI2C();   /*Attente de la disponibilit du bus I2C*/

            if(SSPCON2bits.ACKSTAT) /*Si AckStat == 1, on n'a pas reu d'acquittement de la part de l'esclave*/
               {
                  StopI2C();  /*Envoi de message de fin de la communication I2C*/
                     return information;   /*On renvoie -1, ce qui indique qu'il y a eu un problme*/
               }

         putcI2C(donnee);   /*On envoie la valeur*/
         IdleI2C();   /*Attente de la disponibilit du bus I2C*/

            if(!(SSPCON2bits.ACKSTAT)) /*Si on reoit un acquitement de la part de l'esclave,
                                   on retournera 1  la fin de la fontion*/
               {
                  information = 1;
               }

         StopI2C();    /*Envoi de message de fin de la communication I2C*/

            return information;
   }



int Lire(unsigned char adresse)
   {
      int valeur = (-1);

         IdleI2C();    /*Attente de la disponibilit du bus I2C*/
         StartI2C();    /*Envoi de message de dbut de la communication I2C*/
         IdleI2C();    /*Attente de la disponibilit du bus I2C*/

         putcI2C(adresse); /*Envoi de l'adresse de l'esclave*/
         IdleI2C();   /*Attente de la disponibilit du bus I2C*/
         
            if (SSPCON2bits.ACKSTAT) /*Si AckStat == 1, on n'a pas reu d'acquittement de la part de l'esclave*/
               {
                  StopI2C();  /*Envoi de message de fin de la communication I2C*/
                     return valeur;   /*On renvoie -1, ce qui indique qu'il y a eu un problme*/
               }

         valeur = ReadI2C(); /*Lecture de la valeur qui est reue sur le bus I2C*/

         NotAckI2C();   /*On n'envoie pas d'acquittement car on lit qu'un seul caractre*/

         IdleI2C();   /*Attente de la disponibilit du bus I2C*/

         StopI2C();   /*Envoi de message de fin de la communication I2C*/

            return valeur;   /*On renvoie la valeur lue*/
   }



/*Envoyer une commande  l'afficheur alphanumrique, grce au module  I2C 1  du STM32F100RB 

Paramtre d'entre :

commande - Commande  envoyer.

Valeur de retour :

Aucune.

*/
void EnvoiCommande(unsigned char commande)
	{
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x00));	/*PORTC = ((PORTC & 0xF8) | (0x00));*/	/*RS = 0, R/W = 0, E = 0*/
		Ecrire(AdresseCommande1, commande);	/*PORTD = commande;*/	/*RS = 0, R/W = 0, E = 0*/
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x04));	/*PORTC = ((PORTC & 0xF8) | (0x04));*/ /*RS = 0, R/W = 0, E = 1*/
		Delai(300);
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x00));	/*PORTC = ((PORTC & 0xF8) | (0x00));*/	/*RS = 0, R/W = 0, E = 0*/
	}



/*Placer le curseur de l'afficheur alphanumrique  un endroi spcifi

Paramtre d'entre :

adresse_du_curseur - Adresse, sur l'afficheur alphanumrique, o placer le curseur.

Valeur de retour :

Aucune.

*/
void PlacerCurseur(unsigned char adresse_du_curseur)
	{
		EnvoiCommande(0x80 | adresse_du_curseur);
		AdresseCurseurActuelle = adresse_du_curseur;
	}



/*Envoyer une donne  l'afficheur alphanumrique, grce au module  I2C 1  du STM32F100RB 

Paramtre d'entre :

donnee - Donne  envoyer.

Valeur de retour :

Aucune.

*/
void EnvoiDonnee(unsigned char donnee)
	{
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x01));	/*PORTC = ((PORTC & 0xF8) | (0x01));*/	/*RS = 1*/
		Ecrire(AdresseCommande1, donnee);	/*PORTD = donnee;*/	/*RS = 1*/
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x05));	/*PORTC = ((PORTC & 0xF8) | (0x05));*/	/*RS = 1, E = 1*/
		Delai(3);
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x01));	/*PORTC = ((PORTC & 0xF8) | (0x01));*/	/*RS = 1*/
	}



/*Initialiser l'afficheur alphanumrique

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void InitialisationAfficheur(void)
	{
		int k;
		unsigned char commandes_pour_initialisation[5] = {0x01, 0x38, 0x0C, 0x06, 0x02};
		//PORTC = 0x00;
		//PORTD = 0x00;
			for(k = 0; k < 5; k++)
				{
					EnvoiCommande(commandes_pour_initialisation[k]);
				}
	}



/*Mettre en veille l'afficheur alphanumrique

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void MiseVeilleAfficheur(void)
	{
		int k;
		unsigned char commandes_pour_veille[3] = {0x01, 0x02, 0x08};
			//PORTC = 0x00;
			//PORTD = 0x00;
				for(k = 0; k < 3; k++)
					{
						EnvoiCommande(commandes_pour_veille[k]);
					}
	}



/*Obtenir l'adresse du curseur, sur l'afficheur alphanumrique, grce au module  I2C 1  du STM32F100RB 

Paramtre d'entre :

Aucun.

Valeur de retour :

Adresse du curseur, sur l'afficheur alphanumrique.

*/
unsigned char ObtenirAdresseActuelle(void)
	{
		unsigned char valeur_adresse_obtenue;
			Ecrire(AdresseCommande1, 0xFF);	/*TRISD = 0xFF;*/
			Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x06));	/*PORTC = ((PORTC & 0xF8) | (0x06));*/	/*RS = 0, R/W = 1, E = 1*/
			Delai(3);
				valeur_adresse_obtenue = Lire(AdresseCommande1 | 0x01);	/*valeur_adresse_obtenue = PORTD;*/	/*RS = 0, R/W = 1, E = 1*/
			/*TRISD = 0x00;*/
			Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x02));	/*PORTC = ((PORTC & 0xF8) | (0x02));*/	/*RS = 0, R/W = 1, E = 0*/
				return valeur_adresse_obtenue;
	}



/*Afficher un caractre, sur l'afficheur alphanumrique

Paramtre d'entre :

c - Code ASCII du caractre  afficher.

Valeur de retour :

Aucune.

*/
void AfficheCaractere(unsigned char c)
	{
		EnvoiDonnee(c);
			if(ObtenirAdresseActuelle() == 0x08)
				{
					PlacerCurseur(0x28);
				}
	}



/*Afficher un texte, sur l'afficheur alphanumrique

Paramtre d'entre :

* texte - Chane de caractres, contenant les codes ASCII des caractres  afficher.

Valeur de retour :

Aucune.

*/
void AfficheTexte(unsigned short char * texte)
	{
		int i = 0;
			while (texte[i] != '\0')
				{
					AfficheCaractere(texte[i]);
						i++;
				}
	}



/*Envoyer une ligne d'un caractre spcial, dfini par l'utilisateur,  l'afficheur alphanumrique, grce au module  I2C 1  du STM32F100RB 

Paramtre d'entre :

uoct_special - Correspond au points de l'afficheur  solliciter, pour former la ligne du caractre spcial dfini par l'utilisateur.

Valeur de retour :

Aucune.

*/
void CommandeSpeciale(unsigned char uoct_special)
	{
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x01));	/*PORTC = ((PORTC & 0xF8) | (0x01));*/	/*RS = 1, R/W = 0, E = 0*/
		Ecrire(AdresseCommande1, uoct_special);	/*PORTD = uoct_special;*/	/*RS = 1, R/W = 0, E = 0*/
		Delai(3);
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x05));	/*PORTC = ((PORTC & 0xF8) | (0x05));*/	/*RS = 1, R/W = 0, E = 1*/
		Delai(3);
		Ecrire(AdresseCommande2, ((EtatDesLED << 2 ) | 0x01));	/*PORTC = ((PORTC & 0xF8) | (0x01));*/	/*RS = 1, R/W = 0, E = 0*/
	}



/*Envoyer huit caractres spciaux, dfini par l'utilisateur,  l'afficheur alphanumrique 

Paramtre d'entre :

str_nouveaucaractere - Tableau contenant la dfinition des huit caractres spciaux, dfinis par l'utilisateur.

Valeur de retour :

Aucune.

*/
void ChargeCaracteres(unsigned char str_nouveaucaractere[])
	{
	    int i;
		EnvoiCommande(0x40);
			for(i = 0; i < 56; i++)
				{
					CommandeSpeciale(str_nouveaucaractere[i]);
				}
	    PlacerCurseur(0x00);
	}



/*Convertir un nombre dans une base, et afficher le rsultat de cette conversion, sur l'afficheur alphanumrique 

Paramtres d'entre :

valeur - Nombre  afficher.
base - Base de comptage du nombre  afficher, comprise entre 2 et 16.

Valeur de retour :

Aucune.

*/
void ConvertirEtAfficher(int valeur, unsigned char base)
	{
		unsigned char symboles[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
		int k = 0;
		static unsigned char tampon[8];
			do
				{
					tampon[k]= symboles[valeur % base];
					valeur = valeur / base;
					k++;
				}
		while(valeur > 0);
		if((k == 1) && (base == 16))
			{
				AfficheCaractere('0');
			}
		do
			{
				k--;
				AfficheCaractere(tampon[k]);		
			}
		while(k > 0);
	}



/*La fonction  AllumerUneLED  sert  allumer une LED, on peut allumer une LED !

Paramtre d'entre :

l - Numro de la LED  allumer.

Valeur de retour :

Aucune.

*/
void AllumerUneLED(unsigned char l)
	{
		unsigned char p = 0;
			p = (EtatDesLED & (~(1 << l)));
				Ecrire(AdresseCommande2, (p << 2));
			EtatDesLED = p;
	}



/*La fonction  AllumerUneLED  sert  teindre une LED, on peut teindre une LED !

Paramtre d'entre :

r - Numro de la LED  teindre.

Valeur de retour :

Aucune.

*/
void EteindreUneLED(unsigned char r)
	{
		unsigned char v = 0;
			v = (EtatDesLED | (1 << r));
				Ecrire(AdresseCommande2, (v << 2));
			EtatDesLED = v;	
	}



/*Fonction principale 

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void main(void)
	{
		int i;
		int u;

			/*Configuration horloge interne du pic  32 MHz*/
			OSCCON = 0x70;	/*Pas de prdiviseur => 8MHz*/
			OSCTUNE = 0x40;	/*Activer la PLL * 4 => 32MHz*/

			/*Dsactiver le module CCP*/
			CCP1CON=0;

			/*Dsactiver le convertisseur analogique numerique*/
			ADCON1=0x0F;

			TRISBbits.TRISB0 = 1;
			TRISBbits.TRISB1 = 1;

				OpenI2C(MASTER, SLEW_OFF);

			SSPADD = 99;  /*100KHz sous 40MHz*/
			SSPCON1bits.CKP = 0;
			PIE1bits.SSPIE = 0;
			PIR1bits.SSPIF = 0;

			EtatDesLED = 0x00;

				InitialisationAfficheur();

				EnvoiCommande(0x01);	/*Effacement de l'afficheur LCD*/
				Delai(1000);
				PlacerCurseur(0x00);

				ChargeCaracteres(speciaux);

				AfficheTexte(TableauDeTextes[1]);

				EteindreUneLED(1);
				EteindreUneLED(2);
				EteindreUneLED(3);
				EteindreUneLED(4);
				EteindreUneLED(5);	

					while(1)
						{
							for(i = 0; i < 9; i++)
								{
									for(u = 1; u < 6; u++)
										{
											AllumerUneLED(u);	/*Effet de chenillard, de gauche  droite, sur les 5 LED*/
											Delai(1000);
											EteindreUneLED(u);
										}
									for(u = 5; u > 0; u--)
										{
											AllumerUneLED(u);	/*Effet de chenillard, de droite  gauche, sur les 5 LED*/
											Delai(1000);
											EteindreUneLED(u);
										}
									if((i % 3) == 0)	/*On fait trois aller-retour avec l'effet chenillard, avant de changer le texte affich par l'afficheur alphanumrique*/
										{
											EnvoiCommande(0x01);	/*Effacement de l'afficheur LCD*/
											Delai(1000);
											PlacerCurseur(0x00);
											AfficheCaractere(1);
											Delai(1000);
											AfficheTexte(TableauDeTextes[i / 3]);	/*Changement du texte affich par l'afficheur alphanumrique*/
										}
								}
						}
	}
